#ISSR Workshop - Advanced Data Management in R
#Matthew Denny, mdenny@polsci.umass.edu, 11-01-13

#some preliminaries
rm(list = ls())

#you will need to set this to the location of th folder you downloaded for this workshop
setwd("~/Dropbox/RA_and_Consulting_Work/ISSR_Consulting_Work/Advanced_Data_Management_in_R")

#We are going to be looking at a dataset that comprises 11 sessions on congress and sepcifically the bills cosponsored by senators over that period of time. This is a very complex data management problem.
Congresses <- 11

#1. The assign function lets us name objects usign the paste function in R. This is good for batch reading in data or generating lots of data objects.

#lets begin by loading in some data:
print("Loading Raw Senate Cosponsorship Matrix Data")
for(i in 1:Congresses){
    cur <- 99 + i
    temp <- read.csv(paste(cur,"_senmatrix.txt", sep = ""), stringsAsFactors = F, header = F)
    #we are only going to look at the first 100 bills from each congress
    temp <- temp[,1:100]
    assign(paste("Senate_Raw_Cosponsorship_",cur, sep = ""),temp) #assigns the raw cosponsorship data to a named variable
}


#2. The get() function performs the opposite funtion of the assign function and lets us get objects by their pasted name:

#get a list of all the raw data which we will not care about after we have transformed it so we can remove it later
all_objects <- ls()

print("Transforming Raw data into Cosponsorship Matricies")
for(i in 1:Congresses){
    cur <- 99 + i
    print(paste("Currently on Congress number: ",cur, sep = ""))
    temp <- get(paste("Senate_Raw_Cosponsorship_",cur, sep = ""))
    
    #create a sociomatrix to populate
    num_senators <- length(temp[,1])
    temp_sociomatrix <- matrix(0,ncol = num_senators, nrow = num_senators)
    
    # this is an example of nested looping 
    for(j in 1:length(temp[1,])){#for every bill
        
        #find out who the bill sponsor is (coded as a 1)
        for(k in 1: length(temp[,1])){ #for every Senator
            if(temp[k,j] == 1){
                sponsor <- k
            }  
        }
        
        #find all of the cosponsors
        for(k in 1: length(temp[,1])){ #for every Senator
            if(temp[k,j] == 2){
                temp_sociomatrix[sponsor,k] <- temp_sociomatrix[sponsor,k] + 1
            } 
        }
    }
    
    assign(paste("Senate_Cosponsorship_Matrix_",cur, sep = ""),temp_sociomatrix) #assigns the cosponsorship matrix to a named variable
   
}

#remove all of the raw data
rm(list = all_objects)


#3. define a function 
#so far we have written code which could be turned into a function. A simple example might be a function that counts the total number of cosponsorships in a given congress.

Total_Cosponsorships <- function(cosponsorship_matrix){
    total <- 0
    for(i in 1:length(cosponsorship_matrix[1,])){
        for(j in 1:length(cosponsorship_matrix[,1])){
            total <- total + cosponsorship_matrix[j,i]
        }
    }
    return(total)
}

#now we have to enter the function in the console before we can use it. If you are using RStudio then you will see the function pop up in your workspace.
#try it out!
Total_Cosponsorships(Senate_Cosponsorship_Matrix_100)

#4. We can use functions inside of functions as well. Note that we can set default values for a function by putting an = sign next to the arguemnt name

Cosponsorships_per_Congress <- function(start = 100, end = 110){
    num_cases <- end - start + 1
    vector <- rep(0, times = num_cases)
    for(i in 1:num_cases){
        cur <- start - 1 + i
        #sometimes we want an update of where we are in the process
        print(paste("Currently on Congress Number: ", cur , sep = ""))
        #we can also call a function from inside a function... inside a function... inside a function
        vector[i] <- Total_Cosponsorships(get(paste("Senate_Cosponsorship_Matrix_",cur, sep = "")))
    }
    return(vector)
}

#try it out (note that you do not need to have a )
Cosp_per_Congress <- Cosponsorships_per_Congress()

#plot our work
barplot(Cosp_per_Congress,xlab = "Congress", ylab = "Number of Cosponsorships", names = c(100:110),col = rainbow(11))



#5. try loading in our functions from a source file
source("My_Functions.R")

#test them out
Threshold_100 <- Threshold(1,Senate_Cosponsorship_Matrix_100)
RowSums_100 <- Row_Sums(Senate_Cosponsorship_Matrix_100)


#6. The apply family of functions

# we can get the rowsums using the apply function
RowSums_Apply_100 <- apply(Senate_Cosponsorship_Matrix_100,1, sum) #takes row sums
ColSums_Apply_100 <- apply(Senate_Cosponsorship_Matrix_100,2, sum) #takes column sums
Mat_Plus_10 <- apply(Senate_Cosponsorship_Matrix_100,1, AddTen) #adds ten to every element of the matrix
Mat_Plus_10 <- sapply(Senate_Cosponsorship_Matrix_100, AddTen) #adds ten to every element of the matrix

#6.5 having some fun
library(statnet)

par(mfrow = c(1,1))
colors <- (1:11)
years <- 100:110

netplot <- function(year, color){
    net <- as.network(get(paste("Senate_Cosponsorship_Matrix_",year, sep = "")))
    plot(net, vertex.col = color)
    if(color == 1){
        output <- degree(net)
    }else{
        output <- rbind(output,degree(net))
    }
    Sys.sleep(.2)
    return(output)
}

result <- mapply(netplot, years,colors)

apply(result,sum)

#7. an example with the reshape package
library(reshape)

#generate a toy dataset quality
a <- c(1:100)
b <- rep(c(1:4),25)
c <- rep(c(1:10),10)
d <- runif(100,1,100)
e <- seq(from = 1.01, to = 2, by = 0.01)

data <- as.data.frame(cbind(a,b,c,d,e))
names(data) <- c("id","color","age","taste","quality")

meltdat <- melt(data, id = c("age","color"))

# cast(data, formula, function) 
cast(meltdat, color~variable, mean)
cast(meltdat, age~variable, sum)
cast(meltdat, age~variable, AddTen)
cast(meltdat, age + color~variable, mean)






